@@ -1,3 +1,5 @@
+#!/usr/bin/python
+# -*- coding: utf-8 -*-
 '''
 Part-of-speech (POS) tagging service
 
@@ -8,8 +10,9 @@ from eWRT.ws.rest import RESTClient
 
 POS_URL = "http://voyager.srv.weblyzard.net/ws"
 
+
 class POS(RESTClient):
-    
+
     def __init__(self, url=POS_URL, usr=None, pwd=None):
         '''
         :param url: URL of the jeremia web service
@@ -23,15 +26,15 @@ class POS(RESTClient):
 
         :returns: the corresponding ANNIE compatible annotations
         """
-        return self.execute("pos-tagging", None, { 'text': text, 'lang': lang })
+        return self.execute("pos-tagging", None, {'text': text, 'lang': lang})
 
 
 class POSTest(unittest.TestCase):
-    
+
     def test_POS(self):
         p = POS()
-        print p.pos_tagging('Guten Tag Herr Mayer!', 'de')
-        
+        print(p.pos_tagging('Guten Tag Herr Mayer!', 'de'))
+
 
 if __name__ == '__main__':
     unittest.main()
