@@ -17,16 +17,13 @@ Remove functions:
  - compatibility fixes for namespaces, encodings etc.
  - support for the old POS tags mapping.
 '''
-
-from collections import namedtuple
 import json
 import logging
-from lxml import etree
-from pprint import pprint
-
-import unittest
 import hashlib
 
+from collections import namedtuple
+from lxml import etree
+
 from weblyzard_api.xml_content.parsers.xml_2005 import XML2005
 from weblyzard_api.xml_content.parsers.xml_2013 import XML2013
 from weblyzard_api.xml_content.parsers.xml_deprecated import XMLDeprecated
@@ -36,10 +33,12 @@ SENTENCE_ATTRIBUTES = ('pos_tags', 'sem_orient', 'significance', 'md5sum',
 
 LabeledDependency = namedtuple("LabeledDependency", "parent pos label")
 
+
 class Annotation(object):
-    
-    def __init__(self, annotation_type=None, start=None, end=None, key=None, 
-                 sentence=None, surfaceForm=None, md5sum=None):
+
+    def __init__(self, annotation_type=None, start=None, end=None, key=None,
+                 sentence=None, surfaceForm=None, md5sum=None, sem_orient=None,
+                 preferredName=None):
         self.annotation_type = annotation_type
         self.surfaceForm = surfaceForm
         self.start = start
@@ -47,13 +46,16 @@ class Annotation(object):
         self.key = key
         self.sentence = sentence
         self.md5sum = md5sum
-    
+        self.sem_orient = sem_orient
+        self.preferredName = preferredName
+
     def as_dict(self):
         '''
         :returns: a dictionary representation of the sentence object.
         '''
         return dict((k, v) for k, v in self.__dict__.iteritems() if not k.startswith('_'))
-    
+
+
 class Sentence(object):
     '''
     The sentence class used for accessing single sentences.
@@ -61,7 +63,7 @@ class Sentence(object):
     .. note::
 
         the class provides convenient properties for accessing pos tags and tokens:
-    
+
         * s.sentence: sentence text
         * s.tokens  : provides a list of tokens (e.g. ['A', 'new', 'day'])
         * s.pos_tags: provides a list of pos tags (e.g. ['DET', 'CC', 'NN'])
@@ -76,20 +78,21 @@ class Sentence(object):
             'token': 'tok_list',
             'is_title': 'is_title',
             'dependency': 'dep_tree',
-            }
         }
-    
-    def __init__(self, md5sum=None, pos=None, sem_orient=None, significance=None, 
+    }
+
+    def __init__(self, md5sum=None, pos=None, sem_orient=None, significance=None,
                  token=None, value=None, is_title=False, dependency=None):
-        
-        if not md5sum and value: 
+
+        if not md5sum and value:
             try:
                 m = hashlib.md5()
-                m.update(value.encode('utf-8') if isinstance(value, unicode) else str(value))
+                m.update(value.encode('utf-8')
+                         if isinstance(value, unicode) else str(value))
                 md5sum = m.hexdigest()
-            except Exception, e: 
-                print e
-                
+            except Exception as e:
+                print(e)
+
         self.md5sum = md5sum
         self.pos = pos
         self.sem_orient = sem_orient
@@ -104,7 +107,7 @@ class Sentence(object):
         :returns: a dictionary representation of the sentence object.
         '''
         return dict((k, v) for k, v in self.__dict__.iteritems() if not k.startswith('_'))
-        
+
     def get_sentence(self):
         return self.value
 
@@ -128,7 +131,7 @@ class Sentence(object):
         if isinstance(new_pos_tags, list):
             new_pos_tags = ' '.join(new_pos_tags)
         self.pos = new_pos_tags
-    
+
     def get_pos_tags_list(self):
         '''
         :returns: list of the sentence's POS tags
@@ -138,10 +141,10 @@ class Sentence(object):
         ['PRP', 'ADV', 'NN']
         '''
         return [] if not self.pos_tag_string else self.get_pos_tags()
-    
+
     def set_pos_tags_list(self, pos_tags_list):
         self.set_pos_tags(pos_tags_list)
-    
+
     def get_pos_tags_string(self):
         '''
         :returns: String of the sentence's POS tags
@@ -151,10 +154,10 @@ class Sentence(object):
         'PRP ADV NN'
         '''
         return self.pos
-    
+
     def set_pos_tags_string(self, new_value):
         self.pos = new_value
-    
+
     def get_tokens(self):
         '''
         :returns: an iterator providing the sentence's tokens 
@@ -164,7 +167,7 @@ class Sentence(object):
 
         for token_pos in self.token.split(' '):
             start, end = map(int, token_pos.split(','))
-            #yield self.sentence.decode('utf8')[start:end]
+            # yield self.sentence.decode('utf8')[start:end]
             yield unicode(self.sentence)[start:end]
 
     def get_dependency_list(self):
@@ -183,7 +186,7 @@ class Sentence(object):
             deps = self.dependency.strip().split(' ')
             for index, dep in enumerate(deps):
                 [parent, label] = dep.split(':') if ':' in dep else [dep, None]
-                result.append(LabeledDependency(parent, 
+                result.append(LabeledDependency(parent,
                                                 self.pos_tags_list[index],
                                                 label))
             return result
@@ -241,46 +244,65 @@ class Sentence(object):
         '''
         key_map = self.API_MAPPINGS[version]
         return {key_map[key]: value for key, value in
-                self.as_dict().iteritems() if key in key_map and \
+                self.as_dict().iteritems() if key in key_map and
                 value is not None}
-            
+
     sentence = property(get_sentence, set_sentence)
     pos_tags = property(get_pos_tags, set_pos_tags)
     tokens = property(get_tokens)
     pos_tags_list = property(get_pos_tags_list, set_pos_tags_list)
     pos_tag_string = property(get_pos_tags_string, set_pos_tags_string)
     dependency_list = property(get_dependency_list, set_dependency_list)
-    
+
+
 class XMLContent(object):
-    
-    SUPPORTED_XML_VERSIONS = {XML2005.VERSION: XML2005, 
-                              XML2013.VERSION: XML2013, 
+
+    SUPPORTED_XML_VERSIONS = {XML2005.VERSION: XML2005,
+                              XML2013.VERSION: XML2013,
                               XMLDeprecated.VERSION: XMLDeprecated}
     API_MAPPINGS = {
         1.0: {
             'lang': 'language_id',
+            'xml:lang': 'language_id',
             'title': 'title',
-            }
+            'uri': 'uri'
         }
-    
+    }
+
+    ATTRIBUTE_MAPPING = {'uri': 'uri',
+                         'content_id': 'id',
+                         'title': 'title',
+                         'sentences': 'sentences',
+                         'body_annotations': 'annotations',
+                         'lang': 'xml:lang',
+                         'sentences_map': {'pos': 'pos',
+                                           'token': 'token',
+                                           'value': 'value',
+                                           'md5sum': 'id'},
+                         'annotations_map': {'start': 'start',
+                                             'end': 'end',
+                                             'key': 'key',
+                                             'surfaceForm': 'surfaceForm'
+                                             }}
+
     def __init__(self, xml_content, remove_duplicates=True):
         self.xml_version = None
         self.attributes = {}
         self.sentence_objects = []
         self.titles = []
         self.remove_duplicates = remove_duplicates
-        
+
         self.body_annotations = []
         self.title_annotations = []
         self.features = {}
         self.relations = {}
 
         result = self.parse_xml_content(xml_content, remove_duplicates)
-        
-        if result: 
+
+        if result:
             self.xml_version, self.attributes, self.sentence_objects, \
-            self.title_annotations, self.body_annotations, self.titles, \
-            self.features, self.relations = result
+                self.title_annotations, self.body_annotations, self.titles, \
+                self.features, self.relations = result
         pass
 
     @classmethod
@@ -291,50 +313,50 @@ class XMLContent(object):
     @classmethod
     def parse_xml_content(cls, xml_content, remove_duplicates=True):
         xml_version = cls.get_xml_version(xml_content)
-        
+
         if not xml_version or not xml_content:
             return None
-        
+
         sentence_objects = []
         annotation_objects = []
         parser = cls.SUPPORTED_XML_VERSIONS[xml_version]
-        
+
         attributes, sentences, title_annotations, body_annotations, features, \
             relations = parser.parse(xml_content, remove_duplicates)
-        
-        if 'title' in attributes:
-            titles = [Sentence(value=attributes['title'], is_title=True)]
-        else: 
-            titles = []
-        
+
+        titles = []
         for sentence in sentences:
-            sent_obj = Sentence(**sentence) 
-            
-            if sent_obj.is_title: 
+            sent_obj = Sentence(**sentence)
+
+            if sent_obj.is_title:
                 titles.append(sent_obj)
-            else: 
+            else:
                 sentence_objects.append(sent_obj)
-                
+
+        if len(titles) == 0 and 'title' in attributes:
+            # fall back titles from attributes
+            titles = [Sentence(value=attributes['title'], is_title=True)]
+
         for annotation in body_annotations:
-            annotation_obj = Annotation(**annotation) 
+            annotation_obj = Annotation(**annotation)
             annotation_objects.append(annotation_obj)
         return xml_version, attributes, sentence_objects, title_annotations, \
-                annotation_objects, titles, features, relations
-    
+            annotation_objects, titles, features, relations
+
     @classmethod
     def get_xml_version(cls, xml_content):
-        if not xml_content: 
+        if not xml_content:
             return None
-        
+
         for version, xml_parser in cls.SUPPORTED_XML_VERSIONS.iteritems():
             if xml_parser.is_supported(xml_content):
                 return version
-            
-    def get_xml_document(self, header_fields='all', 
-                         sentence_attributes=SENTENCE_ATTRIBUTES, 
-                         annotations=None,
-                         xml_version=XML2013.VERSION):
 
+    def get_xml_document(self, header_fields='all',
+                         sentence_attributes=SENTENCE_ATTRIBUTES,
+                         annotations=None,
+                         ignore_title=False,
+                         xml_version=XML2013.VERSION):
         '''
         :param header_fields: the header_fields to include
         :param sentence_attributes: sentence attributes to include
@@ -342,18 +364,22 @@ class XMLContent(object):
         :param xml_version: version of the webLyzard XML format to use (XML2005.VERSION, *XML2013.VERSION*)
         :returns: the XML representation of the webLyzard XML object
         '''
-        
-        if not xml_version: 
+
+        if not xml_version:
             xml_version = self.xml_version
 
         if not hasattr(self, 'features'):
             self.features = {}
-            
+
         if not hasattr(self, 'relations'):
             self.relations = {}
-            
-        return self.SUPPORTED_XML_VERSIONS[xml_version].dump_xml(titles=self.titles,
-                                                                 attributes=self.attributes, 
+
+        titles = self.titles
+        if ignore_title:
+            titles = []
+
+        return self.SUPPORTED_XML_VERSIONS[xml_version].dump_xml(titles=titles,
+                                                                 attributes=self.attributes,
                                                                  sentences=self.sentences,
                                                                  annotations=annotations,
                                                                  features=self.features,
@@ -364,92 +390,120 @@ class XMLContent(object):
         if not len(self.sentences):
             return ''
         return '\n'.join([s.value for s in self.sentences if not s.is_title])
-    
+
     @classmethod
     def get_text(cls, text):
-        ''' :returns: the utf-8 encoded text ''' 
+        ''' :returns: the utf-8 encoded text '''
         if isinstance(text, str):
             text = text.decode('utf-8')
         return text
 
     def add_attribute(self, key, value):
-        if not self.attributes: 
+        if not self.attributes:
             self.attributes = {}
         self.attributes[key] = value
 
     def update_attributes(self, new_attributes):
         '''
         Updates the existing attributes with new ones 
-        
+
         :param new_attributes: The new attributes to set.
         :type new_attributes: dict
         '''
-        
+
         # not using dict.update to allow advanced processing
-        
+
         if not new_attributes or not isinstance(new_attributes, dict):
             return
-        
+
         for k, v in new_attributes.iteritems():
             self.attributes[str(k)] = v
 
-    def as_dict(self, mapping=None, 
-                ignore_non_sentence=False, add_titles_to_sentences=False):
+    def update_features(self, new_features):
+        if not new_features or not isinstance(new_features, dict):
+            return
+
+        for k, v in new_features.iteritems():
+            self.features[str(k)] = v
+
+    def update_relations(self, new_relations):
+        if not new_relations or not isinstance(new_relations, dict):
+            return
+
+        for k, v in new_relations.iteritems():
+            self.relations[str(k)] = v
+
+    def as_dict(self, mapping=None, ignore_non_sentence=False,
+                ignore_features=False, ignore_relations=False,
+                add_titles_to_sentences=False):
         ''' convert the XML content to a dictionary.
 
         :param mapping: an optional mapping by which to restrict/rename \
             the returned dictionary
         :param ignore_non_sentence: if true, sentences without without POS tags \
             are omitted from the result
+        :param ignore_features: if true, document features do not get serialized
+        :param ignore_relations: if true, document relations do not get serialized
+        :param add_titles_to_sentences: if true, titles are treated as sentences
         '''
         try:
-            assert mapping, 'got no mapping'
+            if mapping is None:
+                mapping = self.ATTRIBUTE_MAPPING
+
             result = self.apply_dict_mapping(self.attributes, mapping)
-            sentence_attr_name = mapping['sentences'] if 'sentences' in mapping else 'sentences' 
-            
+            sentence_attr_name = mapping['sentences'] if 'sentences' in mapping else 'sentences'
+
             if 'sentences_map' in mapping:
                 result[sentence_attr_name] = []
                 sent_mapping = mapping['sentences_map']
-                
-                if add_titles_to_sentences and len(self.titles): 
+
+                if add_titles_to_sentences and len(self.titles):
                     sentences = self.titles + self.sentences
-                else: 
+                else:
                     sentences = self.sentences
-            
-                for sent in sentences: 
-                    
-                    if ignore_non_sentence and not sent.pos: 
-                            continue
-                    
+
+                for sent in sentences:
+
+                    if ignore_non_sentence and not sent.pos:
+                        continue
+
                     sent_attributes = self.apply_dict_mapping(sent.as_dict(),
                                                               sent_mapping)
                     result[sentence_attr_name].append(sent_attributes)
-                    
+
             annotation_attr_name = mapping['body_annotations'] \
-                    if 'body_annotations' in mapping else 'body_annotations' 
+                if 'body_annotations' in mapping else 'body_annotations'
             if 'annotations_map' in mapping:
                 result[annotation_attr_name] = []
-                annotation_mapping = mapping['annotations_map']            
-                for annotation in self.body_annotations:                     
+                annotation_mapping = mapping['annotations_map']
+                for annotation in self.body_annotations:
                     annotation_attributes = self.apply_dict_mapping(annotation.as_dict(),
-                                                              annotation_mapping)
+                                                                    annotation_mapping)
                     result[annotation_attr_name].append(annotation_attributes)
-        except Exception:
+
+            if not ignore_features and self.features:
+                result['features'] = self.features
+
+            if not ignore_relations and self.relations:
+                result['relations'] = self.relations
+
+        except Exception as e:
             result = self.attributes
-            result.update({'sentences': [sent.as_dict() for sent in self.sentences]})
-        
+            result.update({'sentences': [sent.as_dict()
+                                         for sent in self.sentences]})
+
         return result
 
     @classmethod
     def apply_dict_mapping(cls, attributes, mapping=None):
         result = attributes
-        
+
         if mapping:
-            result = {} 
+            result = {}
             for attr, value in attributes.iteritems():
                 if attr in mapping:
                     result[mapping[attr]] = value
-            
+
         return result
 
     def to_api_dict(self, version=1.0):
@@ -467,13 +521,14 @@ class XMLContent(object):
         for key in self.API_MAPPINGS[version]:
             if key in document_dict:
                 api_dict[self.API_MAPPINGS[version][key]] = \
-                         document_dict[key]
+                    document_dict[key]
         if self.sentences and len(self.sentences) > 0:
             sentences = [s.to_api_dict(version) for s in self.sentences]
             api_dict['sentences'] = sentences
         if self.titles and len(self.titles) > 0:
             for t in self.titles:
-                api_dict['sentences'] = [t.to_api_dict(version)] + api_dict.get('sentences', [])
+                api_dict['sentences'] = [t.to_api_dict(
+                    version)] + api_dict.get('sentences', [])
                 if 'title' not in api_dict:
                     api_dict['title'] = t.value
 #                 elif api_dict['title'] != t.value:
@@ -483,9 +538,12 @@ class XMLContent(object):
         annotations = document_dict.get('annotations', None)
         if annotations:
             api_dict['annotations'] = annotations
-            
-        return api_dict
 
+        if self.features and len(self.features) > 0:
+            api_dict['features'] = self.features
+        if self.relations and len(self.relations) > 0:
+            api_dict['relations'] = self.relations
+        return api_dict
 
     def to_json(self, version=1.0):
         '''
@@ -502,26 +560,26 @@ class XMLContent(object):
     def _get_attribute(self, attr_name):
         ''' ::returns: the attribute for the given name '''
         return self.attributes.get(attr_name, None)
-    
+
     def get_nilsimsa(self):
         return self._get_attribute('nilsimsa')
-    
+
     def get_content_type(self):
         return self._get_attribute('content_type')
-    
+
     def get_title(self):
         return self._get_attribute('title')
-    
+
     def get_lang(self):
         return self._get_attribute('lang')
 
     def get_content_id(self):
-        content_id = self._get_attribute('content_id') 
+        content_id = self._get_attribute('content_id')
         return int(content_id) if content_id else content_id
-     
+
     def get_sentences(self):
         return self.sentence_objects
-    
+
     def update_sentences(self, sentences):
         ''' 
         updates the values of the existing sentences. if the list of 
@@ -533,10 +591,10 @@ class XMLContent(object):
         .. warning:: this function will not add new sentences
         '''
         if not self.sentence_objects:
-            self.sentence_objects = sentences 
+            self.sentence_objects = sentences
         else:
             sentence_dict = dict((sent.md5sum, sent) for sent in sentences)
-            
+
             for sentence in self.sentence_objects:
                 if sentence.md5sum in sentence_dict:
                     new_sentence = sentence_dict[sentence.md5sum]
@@ -545,10 +603,10 @@ class XMLContent(object):
                         if new_value:
                             setattr(sentence, attrib, new_value)
 
-    sentences = property(get_sentences, update_sentences)    
+    sentences = property(get_sentences, update_sentences)
     plain_text = property(get_plain_text)
     nilsimsa = property(get_nilsimsa)
     content_type = property(get_content_type)
     title = property(get_title)
-    lang = property(get_lang)    
-    content_id   = property(get_content_id) 
\ No newline at end of file
+    lang = property(get_lang)
+    content_id = property(get_content_id)
